@@ -15,7 +15,7 @@ use IO::Compress::Lzop::Constants ;
 require Exporter ;
 our ($VERSION, @ISA, @EXPORT_OK, %EXPORT_TAGS, $UnLzopError);
 
-$VERSION = '2.000_13';
+$VERSION = '2.000_14';
 $UnLzopError = '';
 
 @ISA    = qw( Exporter IO::Uncompress::Base );
@@ -263,13 +263,25 @@ sub postBlockChk
 {
     my $self = shift ;
     my $buffer = shift ;
+    my $offset = shift ;
 
     if (*$self->{Strict} && *$self->{LzopData}{Flags} & FLAG_CRC_UNCOMP
             && *$self->{LzopData}{compSize} != *$self->{LzopData}{uncSize} ) {
 
         my $crc ;
-        $crc = crc32($$buffer)   if *$self->{LzopData}{Flags} & F_CRC32_D ;
-        $crc = adler32($$buffer) if *$self->{LzopData}{Flags} & F_ADLER32_D ;
+
+        my $buf = $buffer;
+
+        if ($offset) {
+            my $x = substr($$buffer, $offset);
+            $buf = \$x;
+        }
+
+        $crc = crc32($$buffer)   
+            if *$self->{LzopData}{Flags} & F_CRC32_D ;
+
+        $crc = adler32($$buffer) 
+            if *$self->{LzopData}{Flags} & F_ADLER32_D ;
         
         return $self->saveErrorString(STATUS_ERROR, "CRC error")
             if *$self->{LzopData}{uncCRC} != $crc;
@@ -375,7 +387,6 @@ For writing lzop files/buffers, see the companion module IO::Compress::Lzop.
 
 
 
-
 =head1 Functional Interface
 
 A top-level function, C<unlzop>, is provided to carry out
@@ -549,6 +560,7 @@ TODO
 
 =item C<< MultiStream => 0|1 >>
 
+
 If the input file/buffer contains multiple compressed data streams, this
 option will uncompress the whole lot as a single data stream.
 
@@ -556,6 +568,35 @@ Defaults to 0.
 
 
 
+
+
+=item C<< TrailingData => $scalar >>
+
+Returns the data, if any, that is present immediately after the compressed
+data stream once uncompression is complete. 
+
+This option can be used when there is useful information immediately
+following the compressed data stream, and you don't know the length of the
+compressed data stream.
+
+If the input is a buffer, C<trailingData> will return everything from the
+end of the compressed data stream to the end of the buffer.
+
+If the input is a filehandle, C<trailingData> will return the data that is
+left in the filehandle input buffer once the end of the compressed data
+stream has been reached. You can then use the filehandle to read the rest
+of the input file. 
+
+Don't bother using C<trailingData> if the input is a filename.
+
+
+
+If you know the length of the compressed data stream before you start
+uncompressing, you can avoid having to use C<trailingData> by setting the
+C<InputLength> option.
+
+
+
 =back
 
 
@@ -771,6 +812,7 @@ The default for this option is off.
 
 
 
+
 =back
 
 =head2 Examples
@@ -824,10 +866,10 @@ Usage is
 
 Reads a single line. 
 
-This method fully supports the use of of the variable C<$/>
-(or C<$INPUT_RECORD_SEPARATOR> or C<$RS> when C<English> is in use) to
-determine what constitutes an end of line. Both paragraph mode and file
-slurp mode are supported. 
+This method fully supports the use of of the variable C<$/> (or
+C<$INPUT_RECORD_SEPARATOR> or C<$RS> when C<English> is in use) to
+determine what constitutes an end of line. Paragraph mode, record mode and
+file slurp mode are all supported. 
 
 
 =head2 getc
@@ -999,8 +1041,8 @@ Usage is
     my $status = $z->nextStream();
 
 Skips to the next compressed data stream in the input file/buffer. If a new
-compressed data stream is found, the eof marker will be cleared, C<$.> will
-be reset to 0.
+compressed data stream is found, the eof marker will be cleared and C<$.>
+will be reset to 0.
 
 Returns 1 if a new stream was found, 0 if none was found, and -1 if an
 error was encountered.
@@ -1011,7 +1053,30 @@ Usage is
 
     my $data = $z->trailingData();
 
-Returns any data that 
+Returns the data, if any, that is present immediately after the compressed
+data stream once uncompression is complete. It only makes sense to call
+this method once the end of the compressed data stream has been
+encountered.
+
+This option can be used when there is useful information immediately
+following the compressed data stream, and you don't know the length of the
+compressed data stream.
+
+If the input is a buffer, C<trailingData> will return everything from the
+end of the compressed data stream to the end of the buffer.
+
+If the input is a filehandle, C<trailingData> will return the data that is
+left in the filehandle input buffer once the end of the compressed data
+stream has been reached. You can then use the filehandle to read the rest
+of the input file. 
+
+Don't bother using C<trailingData> if the input is a filename.
+
+
+
+If you know the length of the compressed data stream before you start
+uncompressing, you can avoid having to use C<trailingData> by setting the
+C<InputLength> option in the constructor.
 
 =head1 Importing 
 
@@ -1035,7 +1100,7 @@ Same as doing this
 
 =head1 SEE ALSO
 
-L<Compress::Zlib>, L<IO::Compress::Gzip>, L<IO::Uncompress::Gunzip>, L<IO::Compress::Deflate>, L<IO::Uncompress::Inflate>, L<IO::Compress::RawDeflate>, L<IO::Uncompress::RawInflate>, L<IO::Compress::Bzip2>, L<IO::Uncompress::Bunzip2>, L<IO::Compress::Lzop>, L<IO::Uncompress::AnyInflate>, L<IO::Uncompress::AnyUncompress>
+L<Compress::Zlib>, L<IO::Compress::Gzip>, L<IO::Uncompress::Gunzip>, L<IO::Compress::Deflate>, L<IO::Uncompress::Inflate>, L<IO::Compress::RawDeflate>, L<IO::Uncompress::RawInflate>, L<IO::Compress::Bzip2>, L<IO::Uncompress::Bunzip2>, L<IO::Compress::Lzop>, L<IO::Compress::Lzf>, L<IO::Uncompress::UnLzf>, L<IO::Uncompress::AnyInflate>, L<IO::Uncompress::AnyUncompress>
 
 L<Compress::Zlib::FAQ|Compress::Zlib::FAQ>
 
